/* 

	PLL DESIGNER

    This program assists in the design of a phase-locked-loop (PLL).

	The specific configuration has an active integrator with feedback
	of R2 and C (Zf), and forward component R1 (Zi). The low-pass
	filter function is:

              R2CS + 1         R2        1
    F(s)  =  -----------   =  ----  +  ------
                R1CS           R1       R1CS


	The program allows the user to design the loop by choosing w(nat)
	and zeta (desired), or it allows the user to enter all component
	values and then evaluates the loop parameters.

	Author: Wheeler, Tom  

	Revision History

	None

			*/


#include <stdio.h>
#include <math.h>

#define ESC 27

struct PLL	{
			char date[25];
			char comment[80];
			double r1,r2,c,k_phi,k_v,n_min,n_max,n_avg;
			} design1, design2;


double PI=3.1415927;


char filename[80];	/* Save/Load filename */

/* Input - Output interface */

int main_attr = 7;	/* main attribute */
int main_high = 0x78;	/* main highlight */

int sub_attr = 7;	/* sub-attribute */
int sub_high = 0x78;	/* sub-highlight */



/* Cursor key control values from Window.A */

#define F1 129
#define F2 130
#define F3 131
#define F4 132
#define F5 133
#define F6 134
#define F7 135
#define F8 136
#define F9 137
#define RIGHT 163
#define LEFT 162
#define UP 160
#define DOWN 164
#define PGUP 161
#define PGDN 166
#define HOME 159
#define END 165
#define ENTER 13
#define DEL 167
#define INS 168
#define CR 13
#define BACK 08
#define ESC 27
#define TAB 9


void set_comment()
{

window_open(0,19,79,23,2,sub_attr,0);
printf("Enter or edit your one-line design comment. ENTER accepts.\n\n>");
edlin(design1.comment,73);
close_window();
}


void save_design()
{
int result;
FILE *f1;

window_open(15,5,64,15,2,sub_attr,0);

printf("   *** SAVE/LOAD Design ***\n\n");
printf("1) SAVE existing design to disk file.\n2) LOAD old design from disk.\n\n");
printf("Choose?");
result = wait_key();

if (result=='1')
	{
	printf("       *** SAVE DESIGN ***\n\n");
	printf("Save as...\n\n>");
	result = edlin(filename, 40);

	if (result == -ESC)	{
						close_window();
						return;
						}


	f1 = fopen( filename,"w");

	if (f1 == NULL)		{
						error_window("! ERROR...cannot write file");
						close_window();
						return;
						}

	printf("\n\nWriting %s...", filename);

	fwrite( &design1 , sizeof( design1 ), 1, f1);

	fclose(f1);

	close_window();
	return;
	}

if (result=='2')
	{
	printf("       *** LOAD DESIGN ***\n\n");
	printf("Load from...\n\n>");
	result = edlin(filename, 40);

	if (result == -ESC)	{
						close_window();
						return;
						}


	f1 = fopen( filename,"r");

	if (f1 == NULL)		{
						error_window("! ERROR...cannot read file");
						close_window();
						return;
						}

	printf("\n\nReading %s...", filename);

	fread( &design1 , sizeof( design1 ), 1, f1);

	fclose(f1);

	close_window();
	return;
	}

close_window();
}



void bad_command()
{
error_window("    *** INVALID COMMAND ***");
}

/* Error routine */

void error_window(message)
char *message;
{
window_open(20,10,59,20,2,sub_attr,0);
printf("%s\n\n   Press any key to continue:",message);
wait_key();
close_window();
}



/* This routine uses the line-editor to input a float value. */


double get_float()
{
char temp[40];

*temp = 0;

edlin( temp, 15 );

return( (double) atof( temp ) );
}


double edit_float( value )
double value;
{
char temp[40];

sprintf(temp,"%.3lf", value);
edlin( temp,15);
return( (double) atof( temp ) );
}

/* ---------- Command routines ----------- */

double set_gamma(gamma)
double gamma;
{

window_open(4,4,76,16,1,sub_attr,0);

printf("          : SET GAMMA @ Nmid :\n\n");
printf("This procedure will adjust the percentage of the impulse\n");
printf("response after which the system will be considered settled.\n");
printf("Most designers consider 10%% a good starting point. Lowering\n");
printf("this percentage increases PERCEIVED settling time, but has\n");
printf("no effect on circuit parameters.\n\n");

printf("What percentage for settling?\n>");

gamma=edit_float(gamma*100)/100.;

if (gamma <= 0) gamma = .1;
if (gamma > 1) gamma = 1;

close_window();

return(gamma);
}

void set_damping()
{
double zeta,wn;

window_open(4,4,76,14,1,sub_attr,0);

printf("          : SET DAMPING @ Nmid :\n\n");
printf("This procedure will compute the value of R2 that\n");
printf("will produce a desired loop damping factor at the\n");
printf("average or median N divisor value.\n\n");

printf("What damping factor is desired?\n>");

zeta=design1.r2*sqrt(design1.k_phi*design1.k_v*design1.c/(4*design1.n_avg*design1.r1));

zeta=edit_float(zeta);

if (zeta >= 1)	printf("\n! WARNING: Unstable loop will result! \n\n");

wn=sqrt(design1.k_v*design1.k_phi / (design1.n_min*design1.r1*design1.c) );
design1.r2 = zeta / sqrt(design1.k_phi*design1.k_v*design1.c/(4*design1.n_avg*design1.r1));

close_window();
}


void scratch_design()
{
double zeta;
double fref,wn;

window_open(4,4,76,24,1,sub_attr,0);

printf("             :  Design PLL from Scratch :\n\n");

printf("This routine will compute the values for R2 and C in order\n");
printf("to satisfy loop parameters. This design procedure is based upon\n");
printf("setting the loop natural frequency to Fref/100.\n\n");

printf("Enter value for R1 (ENTER to accept)\n>");
design1.r1 = edit_float( design1.r1);

printf("\n\nEnter value for N(minimum)  (ENTER to accept)\n>");
design1.n_min = floor( edit_float( design1.n_min ) );

printf("\n\nEnter value for N(maximum)  (ENTER to accept)\n>");
design1.n_max = floor( edit_float( design1.n_max ) );

design1.n_avg = (design1.n_min + design1.n_max) / 2.;

printf("\nEnter value for K_phi (PD) (ENTER to accept)\n>");
design1.k_phi = edit_float( design1.k_phi);

printf("\nEnter value for K_v (VCO) (ENTER to accept)\n>");
design1.k_v = edit_float( design1.k_v);

zeta = 0.5;

printf("\n\nEnter value for loop damping factor @ Nmid?\n>");
zeta = edit_float( zeta );

printf("\n\nWhat is the reference frequency in Hz?\n>");
fref = get_float();

printf("\nEnter desired loop natural frequency @ Nmid.\n");
printf("(%.1lf Hz recommended)\n>", fref/100.);

wn = edit_float( fref/100.)*2*PI;

/* Protection against zero divide */

if (wn==0) wn=377;

design1.c = design1.k_phi*design1.k_v/(design1.n_avg*design1.r1*wn*wn);
design1.r2 = zeta / sqrt(design1.k_phi*design1.k_v*design1.c/(4*design1.n_avg*design1.r1));

close_window();
}



void set_c()
{
double temp;

window_open(4,15,60,20,1,sub_attr,0);

printf(": Filter Capacitor value :\n\n");
printf("Enter new value; or press [RETURN] to\n");
printf("accept the current setting:\n\n");
printf("C (uF)  >");
temp=edit_float( design1.c * 1e6 );

if ( temp == 0)	{
				error_window("! ERROR\nC MUST be non-zero!\n");
				close_window();
				return;
				}

design1.c = temp/1e6;
close_window();
}

void set_r1()
{
double temp;

window_open(4,15,60,20,1,sub_attr,0);

printf(": R1 resistor value :\n\n");
printf("Enter new value; or press [RETURN] to\n");
printf("accept the current setting:\n\n");
printf(">");
temp=edit_float( design1.r1 );

if ( temp == 0)	{
				error_window("! ERROR\nR1 MUST be non-zero!\n");
				close_window();
				return;
				}

design1.r1 = temp;
close_window();
}

void set_r2()
{
double temp;

window_open(4,15,60,20,1,sub_attr,0);

printf(": R2 resistor value :\n\n");
printf("Enter new value; or press [RETURN] to\n");
printf("accept the current setting:\n\n");
printf(">");
temp=edit_float( design1.r2 );

if ( temp == 0)	{
				error_window("! ERROR\nR2 MUST be non-zero!\n");
				close_window();
				return;
				}

design1.r2 = temp;
close_window();
}

void set_nmin()
{
long int d;

window_open(4,5,60,15,1,sub_attr,0);

printf(": Feedback Divisor MINIMUM value :\n\n");
printf("Enter new value; or press [RETURN] to\n");
printf("accept the current setting:\n\n");
printf(">");
d = (long) edit_float( design1.n_min );

if ( d == 0)	{
				error_window("! ERROR\nDivisors MUST be non-zero!\n");
				close_window();
				return;
				}

design1.n_min = (double) d;
close_window();
}

void set_nmax()
{
long int d;

window_open(4,9,60,19,1,sub_attr,0);

printf(": Feedback Divisor MAXIMUM value :\n\n");
printf("Enter new value; or press [RETURN] to\n");
printf("accept the current setting:\n\n");
printf(">");
d = (long) edit_float( design1.n_max );

if ( d == 0)	{
				error_window("! ERROR\nDivisors MUST be non-zero!\n");
				close_window();
				return;
				}

design1.n_max = (double) d;
close_window();
}




void set_kphi()
{

window_open(4,5,60,12,1,sub_attr,0);

printf(": Phase Detector Sensitivity (Volts/rad) :\n\n");
printf("Enter new value; or press [RETURN] to\n");
printf("accept the current setting:\n\n");
printf(">");

design1.k_phi = edit_float( design1.k_phi);

close_window();
}



void set_kv()
{
int r;
double f1,f2,v1,v2;


window_open(4,4,76,14,1,sub_attr,0);

printf(": Setting Kv (VCO Sensitivity Constant) :\n\n");
printf("1) Calculate new Kv when frequencies are in Hz\n");
printf("2) Directly enter new Kv value (rad/s/v)\n\n");
printf("[ESC] Cancel command and return to menu\n\n");
printf("Choose?");

r=wait_key();

if (r==ESC)	{
			close_window();
			return;
			}

cls_window(sub_attr);

switch(r)	{
		case '1':
			printf("This procedure calculates Kv by reading two VCO\n");
			printf("frequencies, and two corresponding VCO control voltages.\n");
			printf("VCO linearity is assumed.\n\n");
			printf("\nVCO lower frequency (Hz) ?");
			f1=get_float();
			printf("\nVCO control voltage at this frequency (V)?");
			v1=get_float();
			printf("\n\nVCO upper frequency (Hz) ?");
			f2=get_float();
			printf("\nVCO control voltage at this frequency (V)?");
			v2=get_float();

			if ((v2-v1)==0)
			{
			error_window("! ERROR\n\nControl Voltages must be different.\n");
			break;
			}

			design1.k_v=2*PI*(f2-f1)/(v2-v1);
			close_window();
			return;


		case '2':

			printf("Direct Kv Entry...\n\n");
			printf("Enter new value for Kv(rad/s/v)?");
			design1.k_v = edit_float(design1.k_v);
			close_window();
			return;


		}

		close_window();
		return;			
}

void main()
{
char c;
int r;

double zeta,alpha,gamma,wn,ts;	/* Values @ mid-value of N */


exec("","");
r=init_window();	/* init window system */

if (r)	{
		printf("Insufficient system memory or\ninvalid display mode.\n");
		exit(1);
		}


/* Determine if color card; set attributes for COLOR if so */

	if (vidmode() != 7)	{
						main_attr=0x1f;
						main_high=0x79;
						sub_attr=0x4f;
						sub_high=0x7c;
						}
							

window_open(0,0,79,24,1,main_attr,0);
set_int10_hook(1);
cls_window( main_attr );



	/* Data for testing */

	design1.k_phi = 0.12;
	design1.k_v = 11e6;
	design1.n_max = 1080;
	design1.n_min = 880;
	design1.c = 2.1e-6;
	design1.r2 = 2285;
	design1.r1 = 4700;
	gamma = 0.1;	/* Initial 10% settling criterion */
	dates(design1.date);
	strcpy(design1.comment, "User Comment");


for(;;)
	{
	cls_window( main_attr );
	printf("                         PLL Designer Main Menu\n");
	printf("                         (c) 1996 Tom A. Wheeler\n\n");

	design1.n_avg = (design1.n_min + design1.n_max) / 2.;

	printf("1)  Kv   = %.1lf rad/sec/Volt\n", design1.k_v );
	printf("2)  Kphi = %.5lf Volt/rad\n", design1.k_phi );
	printf("3)  Nmin = %.1lf\n",design1.n_min);
	printf("    Loop Natural Freq @ Nmin = %.2lf Hz\n",
sqrt(design1.k_v*design1.k_phi / (design1.n_min*design1.r1*design1.c) ) / (2*PI) );
	printf("    Damping Factor @ Nmin = %.5lf\n",
design1.r2*sqrt(design1.k_phi*design1.k_v*design1.c/(4*design1.n_min*design1.r1)));

	printf("\n4)  Nmax = %.1lf\n",design1.n_max);
	printf("    Loop Natural Freq @ Nmax = %.2lf Hz\n",
sqrt(design1.k_v*design1.k_phi / (design1.n_max*design1.r1*design1.c) ) / (2*PI));
	printf("    Damping Factor @ Nmax = %.5lf\n",
design1.r2*sqrt(design1.k_phi*design1.k_v*design1.c/(4*design1.n_max*design1.r1)));

	printf("\n5)  Damping Factor @ Navg = %.5lf\n",

	zeta=design1.r2*sqrt(design1.k_phi*design1.k_v*design1.c/(4*design1.n_avg*design1.r1)));

/* natural frequency @ mid-value of N */

	wn=sqrt(design1.k_v*design1.k_phi / (design1.n_avg*design1.r1*design1.c) );
	alpha=wn*zeta;	/* alpha factor for exponential decay, system impulse */
	ts=-log(gamma)/alpha;


	printf("\n6)  R1 = %.1lf Ohms\n",design1.r1);
	printf("7)  R2 = %.1lf Ohms\n",design1.r2);
	printf("8)  C  = %lf uF\n",design1.c*1e6);
	printf("9)  %s\n", design1.comment);
    printf("D)  Design loop from scratch\n");
    printf("$)  Save or load design from disk file\n");

	printf("\nCommand [ESC] to quit >");

	setcursor_window(41,3);
	printf("         STEP RESPONSE @ Nmid ");
	setcursor_window(41,5);
	printf("%%)  Percent settled to %.1lf %%", 100*gamma);
	setcursor_window(41,6);
	printf("    Settling time = %.2lf mS", 1000*ts);
	setcursor_window(41,7);
	printf("    Damped Frequency = %.2lf Hz",
				sqrt(wn*wn - alpha*alpha)/(2*PI) );
	c=toupper(wait_key());

	switch(c)	{
				case '1':	set_kv();
							break;

				case '2':	set_kphi();
							break;

				case '3':	set_nmin();
							break;

				case '4':	set_nmax();
							break;

				case '5':	set_damping();
							break;

				case '6':	set_r1();
							break;

				case '7':	set_r2();
							break;

				case '8':	set_c();
							break;


				case '9':	set_comment();
							break;

				case '$':	save_design();
							break;

				case 'D':	scratch_design();
							break;

				case '%':	gamma=set_gamma(gamma);
							break;

				}


	if (c==ESC)	{
				close_all();
				exit(0);
				}

	}

}

